@@ -6,7 +6,9 @@ function Style() {
 	this.idUrl = null;
 	this.updateUrl = null;
 	this.md5Url = null;
-	this.appliedUrl = null;
+	this.originalCode = null;
+	this.originalMd5 = null;
+	this.appliedInfo = null;
 	this.lastSavedCode = null;
 	this.applyBackgroundUpdates = null;
 	this.mode = this.CALCULATE_META | this.REGISTER_STYLE_ON_CHANGE;
@@ -20,8 +22,7 @@ function Style() {
  
 
 	this.previewOn = false;
-	//whether the applied url is yet to be calculated
-	this.appliedUrlToBeCalculated = false;
+	this.appliedInfoToBeCalculated = false;
 }
 Style.prototype = {
 
@@ -141,7 +142,7 @@ Style.prototype = {
 	/*
 		stylishStyle instance methods
 	*/
-	init: function(url, idUrl, updateUrl, md5Url, name, code, enabled, originalCode, applyBackgroundUpdates) {
+	init: function(url, idUrl, updateUrl, md5Url, name, code, enabled, originalCode, originalMd5, applyBackgroundUpdates) {
 		//the mode may contain a flag that indicates that this is a load rather than a new style
 		var shouldRegister;
 		if (this.mode & this.INTERNAL_LOAD_EVENT) {
@@ -150,7 +151,7 @@ Style.prototype = {
 		} else {
 			shouldRegister = this.shouldRegisterOnChange()
 		}
-		this.initInternal(url, idUrl, updateUrl, md5Url, name, code, enabled, originalCode, shouldRegister, applyBackgroundUpdates);
+		this.initInternal(url, idUrl, updateUrl, md5Url, name, code, enabled, originalCode, originalMd5, shouldRegister, applyBackgroundUpdates);
 	},
 
 	get name() {
@@ -158,8 +159,8 @@ Style.prototype = {
 	},
 
 	set name(name) {
-		//reference appliedUrl to make sure it has been calculated before we change the name
-		this.appliedUrl;
+		//reference appliedInfo to make sure it has been calculated before we change the name
+		this.appliedInfo;
 		this._name = name;
 	},
 
@@ -230,9 +231,9 @@ Style.prototype = {
 			that.bind(statement, name, value);
 		}
 		if (this.id == 0) {
-			statement = connection.createStatement("INSERT INTO styles (`url`, `idUrl`, `updateUrl`, `md5Url`, `name`, `code`, `enabled`, `originalCode`, `applyBackgroundUpdates`) VALUES (:url, :idUrl, :updateUrl, :md5Url, :name, :code, :enabled, :originalCode, :applyBackgroundUpdates);");
+			statement = connection.createStatement("INSERT INTO styles (`url`, `idUrl`, `updateUrl`, `md5Url`, `name`, `code`, `enabled`, `originalCode`, `applyBackgroundUpdates`, `originalMd5`) VALUES (:url, :idUrl, :updateUrl, :md5Url, :name, :code, :enabled, :originalCode, :applyBackgroundUpdates, :originalMd5);");
 		} else {
-			statement = connection.createStatement("UPDATE styles SET `url` = :url, `idUrl` = :idUrl, `updateUrl` = :updateUrl, `md5Url` = :md5Url, `name` = :name, `code` = :code, `enabled` = :enabled, `originalCode` = :originalCode, `applyBackgroundUpdates` = :applyBackgroundUpdates WHERE `id` = :id;");
+			statement = connection.createStatement("UPDATE styles SET `url` = :url, `idUrl` = :idUrl, `updateUrl` = :updateUrl, `md5Url` = :md5Url, `name` = :name, `code` = :code, `enabled` = :enabled, `originalCode` = :originalCode, `applyBackgroundUpdates` = :applyBackgroundUpdates, `originalMd5` = :originalMd5 WHERE `id` = :id;");
 			b("id", this.id);
 		}
 
@@ -263,6 +264,7 @@ Style.prototype = {
 		b("code", this.code);
 		b("enabled", this.enabled);
 		b("applyBackgroundUpdates", this.applyBackgroundUpdates);
+		b("originalMd5", this.originalMd5);
 
 		try {
 			statement.execute();
@@ -410,6 +412,9 @@ Style.prototype = {
 	},
 
 	get md5() {
+		if (this.originalMd5 != null) {
+			return this.originalMd5;
+		}
 		//https://developer.mozilla.org/en/nsICryptoHash#Computing_the_Hash_of_a_String
 		var converter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
 		converter.charset = "UTF-8";
@@ -476,7 +481,7 @@ Style.prototype = {
 		}
 	},
 
-	applyUpdate: function() {
+	applyUpdate: function(observer) {
 		var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
 		observerService.notifyObservers(this, "stylish-style-update-start", null);
 
@@ -484,6 +489,9 @@ Style.prototype = {
 
 		function notifyDone(result) {
 			observerService.notifyObservers(that, "stylish-style-update-done", result);
+			if (observer) {
+				observer.observe(that, "stylish-style-update-done", result);
+			}
 		}
 
 		function handleFailure() {
@@ -495,11 +503,16 @@ Style.prototype = {
 				notifyDone("update-failure");
 				return;
 			}
-			that.code = code;
 			//we're back to being in sync
 			that.originalCode = code;
-			that.save("update");
-			notifyDone("update-success");
+			that.code = code;
+			that.downloadMd5(that.md5Url, function(md5Sum) {
+				if (md5Sum != null) {
+					that.originalMd5 = md5Sum;
+				}
+				that.save("update");
+				notifyDone("update-success");
+			});
 		}
 		if (this.updateUrl) {
 			this.download(this.updateUrl, handleSuccess, handleFailure);
@@ -677,21 +690,32 @@ Style.prototype = {
 		var dataUrl = this.dataUrl;
 		if (!dataUrl)
 			return;
-		this.appliedUrl = dataUrl;
-		this.sss.loadAndRegisterSheet(this.appliedUrl, this.sss.AGENT_SHEET);
+		var registrationMethod = this.calculateRegistrationMethod();
+		// Save what we applied so we know what to remove later
+		this.appliedInfo = [dataUrl, registrationMethod];
+		this.sss.loadAndRegisterSheet(dataUrl, registrationMethod);
 	},
 
 	unregister: function() {
-		var unregisterUrl = this.shouldUnregisterOnLoad() ? this.dataUrl : this.appliedUrl;
-		if (unregisterUrl == null) {
+		var unregisterUrl;
+		var unregisterMethod;
+		if (this.shouldUnregisterOnLoad()) {
+			unregisterUrl = this.dataUrl;
+			unregisterMethod = this.calculateRegistrationMethod();
+		} else if (this.appliedInfo == null) {
+			// not registered in the first place
 			return;
+		} else {
+			unregisterUrl = this.appliedInfo[0];
+			unregisterMethod = this.appliedInfo[1];
 		}
-		if (this.sss.sheetRegistered(unregisterUrl, this.sss.AGENT_SHEET))
-			this.sss.unregisterSheet(unregisterUrl, this.sss.AGENT_SHEET);
+		
+		if (this.sss.sheetRegistered(unregisterUrl, unregisterMethod))
+			this.sss.unregisterSheet(unregisterUrl, unregisterMethod);
 		// ignore unregistered styles if stylish isn't on
 		else if (this.stylishOn)
 			Components.utils.reportError("Stylesheet is supposed to be unregistered, but it's not registered in the first place.");
-		this.appliedUrl = null;
+		this.appliedInfo = null;
 	},
 
 	bind: function(statement, name, value) {
@@ -745,16 +769,16 @@ Style.prototype = {
 		}
 	},
 
-	get appliedUrl() {
-		if (this.appliedUrlToBeCalculated) {
-			this.appliedUrl = this.dataUrl;
-			this.appliedUrlToBeCalculated = false;
+	get appliedInfo() {
+		if (this.appliedInfoToBeCalculated) {
+			this.appliedInfo = [this.dataUrl, this.calculateRegistrationMethod()];
+			this.appliedInfoToBeCalculated = false;
 		}
-		return this._appliedUrl;
+		return this._appliedInfo;
 	},
 
-	set appliedUrl(url) {
-		this._appliedUrl = url;
+	set appliedInfo(info) {
+		this._appliedInfo = info;
 	},
 
 	findSql: function(sql, parameters, mode, connection) {
@@ -783,7 +807,7 @@ Style.prototype = {
 					style.mode = mode;
 				// since we can't call initInternal because we're not "inside" the new style, we'll pass a secret flag in the mode
 				style.mode += this.INTERNAL_LOAD_EVENT
-				style.init(e("url"), e("idUrl"), e("updateUrl"), e("md5Url"), e("name"), e("code"), e("enabled"), e("originalCode"), e("applyBackgroundUpdates"));
+				style.init(e("url"), e("idUrl"), e("updateUrl"), e("md5Url"), e("name"), e("code"), e("enabled"), e("originalCode"), e("originalMd5"), e("applyBackgroundUpdates"));
 				style.id = e("id");
 				styles.push(style);
 				styleMap[style.id] = style;
@@ -838,8 +862,8 @@ Style.prototype = {
 	},
 
 	setCode: function(code, shouldRegister) {
-		//reference appliedUrl to make sure it has been calculated before we change the code
-		this.appliedUrl;
+		//reference appliedInfo to make sure it has been calculated before we change the code
+		this.appliedInfo;
 		//save the last saved code in case we have to revert
 		if (!this.lastSavedCode && this.code && this.id)
 			this.lastSavedCode = this.code;
@@ -851,7 +875,7 @@ Style.prototype = {
 		this.calculateInternalMeta();
 	},
 
-	initInternal: function(url, idUrl, updateUrl, md5Url, name, code, enabled, originalCode, shouldRegister, applyBackgroundUpdates) {
+	initInternal: function(url, idUrl, updateUrl, md5Url, name, code, enabled, originalCode, originalMd5, shouldRegister, applyBackgroundUpdates) {
 		this.url = url;
 		this.idUrl = idUrl;
 		this.updateUrl = updateUrl;
@@ -859,9 +883,10 @@ Style.prototype = {
 		this.name = name;
 		this._enabled = enabled;
 		this.originalCode = originalCode;
+		this.originalMd5 = originalMd5;
 		this.setCode(code, shouldRegister);
 		if (!shouldRegister && this.enabled) {
-			this.appliedUrlToBeCalculated = true;
+			this.appliedInfoToBeCalculated = true;
 		}
 		if (this.shouldUnregisterOnLoad()) {
 			this.unregister();
@@ -918,7 +943,9 @@ Style.prototype = {
 					successCallback(request.responseText, contentType);
 				} else {
 					Components.utils.reportError("Download of '" + url + "' resulted in status " + request.status);
-					failureCallback();
+					if (failureCallback) {
+						failureCallback();
+					}
 				}
 			}
 		}, false);
@@ -956,6 +983,31 @@ Style.prototype = {
 
 	unescapeRegexLiterals: function(s) {
 		return s.replace(/\\/g, "");
+	},
+
+	calculateRegistrationMethod: function() {
+		// Default to AUTHOR_SHEET if available, AGENT_SHEET if not or specifically chosen with a comment like /* AGENT_SHEET */
+		if (!("AUTHOR_SHEET" in this.sss) || /\/\*\s*AGENT_SHEET\s*\*\//.test(this.code)) {
+			return this.sss.AGENT_SHEET;
+		}
+		return this.sss.AUTHOR_SHEET;
+	},
+
+	downloadMd5: function(md5Url, callback) {
+		if (!md5Url) {
+			callback(null);
+			return;
+		}
+		this.download(md5Url,
+			function(text, contentType) {
+				if (text.length == 32) {
+					callback(text);
+				} else {
+					Components.utils.reportError("Invalid md5 at URL '" + md5Url + "'.");
+					callback(null)
+				}
+			}, null
+		);
 	}
 
 };
